@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 1.0.
+ * This file is part of CVXOPT version 1.1.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ matrix * Matrix_NewFromArrayStruct(PyObject *, int, int *) ;
 extern PyTypeObject spmatrix_tp ;
 spmatrix * SpMatrix_New(int_t, int_t, int, int ) ;
 spmatrix * SpMatrix_NewFromMatrix(matrix *, int) ;
-spmatrix * SpMatrix_NewFromSpMatrix(spmatrix *, int, int) ; 
+spmatrix * SpMatrix_NewFromSpMatrix(spmatrix *, int) ; 
 spmatrix * SpMatrix_NewFromIJV(matrix *, matrix *, matrix *, 
     int_t, int_t, int, int) ;
 void free_ccs(ccs *) ;
@@ -58,8 +58,6 @@ extern int (*sp_syrk[])(char, char, number, void *, number,
 const int  E_SIZE[] = { sizeof(int_t), sizeof(double), sizeof(complex) };
 const char TC_CHAR[][2] = {"i","d","z"} ;
 
-PyObject *base_mod;
-
 /* 
  *  Helper routines and definitions to implement type transparency.
  */
@@ -348,7 +346,7 @@ PyObject * base_axpy(PyObject *self, PyObject *args, PyObject *kwrds)
   number a;
   char *kwlist[] = {"x", "y", "alpha", "partial", NULL};
   
-  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OO|OO", kwlist, 
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OO|OO:axpy", kwlist, 
 	  &x, &y, &ao, &partial)) return NULL;
   
   if (!Matrix_Check(x) && !SpMatrix_Check(x)) err_mtrx("x");
@@ -369,7 +367,7 @@ PyObject * base_axpy(PyObject *self, PyObject *args, PyObject *kwrds)
 	MAT_BUF(y), (int *)&One[INT]);
   }
   else {
-    
+
     void *z = NULL;
     if (sp_axpy[id]((ao ? a : One[id]), 
 	    Matrix_Check(x) ? MAT_BUF(x): ((spmatrix *)x)->obj,  
@@ -427,7 +425,7 @@ PyObject* base_gemm(PyObject *self, PyObject *args, PyObject *kwrds)
   char *kwlist[] = {"A", "B", "C", "transA", "transB", "alpha", "beta",
 		    "partial", NULL};
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|ccOOO", 
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|ccOOO:gemm", 
 	  kwlist, &A, &B, &C, &transA, &transB, &ao, &bo, &partial))
     return NULL;
 
@@ -534,7 +532,7 @@ static PyObject* base_gemv(PyObject *self, PyObject *args, PyObject *kwrds)
 		    "incx", "incy", "offsetA", "offsetx",
 		    "offsety", NULL};
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|cOOiiiiiii", 
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|cOOiiiiiii:gemv", 
 	  kwlist, &A, &x, &y, &trans, &ao, &bo, &m, &n, &ix, &iy, 
 	  &oA, &ox, &oy))
     return NULL;
@@ -629,7 +627,7 @@ static PyObject* base_syrk(PyObject *self, PyObject *args, PyObject *kwrds)
   char *kwlist[] = {"A", "C", "uplo", "trans", "alpha", "beta", "partial",
 		    NULL};
   
-  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OO|ccOOO", kwlist, 
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OO|ccOOO:syrk", kwlist, 
 	  &A, &C, &uplo, &trans, &ao, &bo, &partial))
     return NULL;
   
@@ -722,8 +720,8 @@ static PyObject* base_symv(PyObject *self, PyObject *args, PyObject *kwrds)
   char *kwlist[] = {"A", "x", "y", "uplo", "alpha", "beta", "n",
 		    "incx", "incy", "offsetA", "offsetx", "offsety", NULL};
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|cOOiiiiii", kwlist, &A, 
-	  &x, &y, &uplo, &ao, &bo, &n, &ix, &iy, &oA, &ox, &oy))
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "OOO|cOOiiiiii:symv", 
+	  kwlist, &A, &x, &y, &uplo, &ao, &bo, &n, &ix, &iy, &oA, &ox, &oy))
     return NULL;
 
   if (!Matrix_Check(A) && !SpMatrix_Check(A)) 
@@ -782,6 +780,24 @@ static PyObject* base_symv(PyObject *self, PyObject *args, PyObject *kwrds)
 
 spmatrix * sparse_concat(PyObject *L, int id_arg) ;
 
+static char doc_sparse[] =
+  "Constructs a sparse block matrix.\n\n"
+  "sparse(x, tc = None)\n\n"
+  "PURPOSE\n"
+  "Constructs a sparse block matrix from a list of block matrices.  If a\n"
+  "single matrix is given as argument,  then the matrix is converted to\n"
+  "sparse format, optionally with a different typcode.  If a single list\n"
+  "of subblocks is specified, then a block column matrix is created;\n"
+  "otherwise when a list of lists is specified, then the inner lists\n"
+  "specify the different block-columns.  Each block element must be either\n"
+  "a dense or sparse matrix, or a scalar,  and dense matrices are converted\n"
+  "to sparse format by removing 0 elements.\n\n"
+  "ARGUMENTS\n"
+  "x       a single matrix, or a list of matrices and scalars, or a list of\n"
+  "        lists of matrices and scalars\n\n"
+  "tc      typecode character 'd' or 'z'.";
+  
+
 static PyObject *
 sparse(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
@@ -789,7 +805,7 @@ sparse(PyTypeObject *type, PyObject *args, PyObject *kwds)
   static char *kwlist[] = { "x", "tc", NULL};
    char tc = 0;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|c:matrix", kwlist, 
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|c:sparse", kwlist, 
 	  &Objx, &tc))
     return NULL;
 
@@ -864,13 +880,15 @@ spdiag(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
   PyObject *diag = NULL, *Dk;
   static char *kwlist[] = { "diag", NULL};
-
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O", kwlist, &diag))
+  
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:spdiag", kwlist, &diag))
     return NULL;
   
-  if ((!PyList_Check(diag) && !Matrix_Check(diag)) ||
+  if ((!PyList_Check(diag) && !Matrix_Check(diag) && !SpMatrix_Check(diag)) ||
       (Matrix_Check(diag) && 
-	  (MAT_LGT(diag) != MAX(MAT_NROWS(diag),MAT_NCOLS(diag)))))
+	  (MAT_LGT(diag) != MAX(MAT_NROWS(diag),MAT_NCOLS(diag)) )) || 
+      (SpMatrix_Check(diag) && 
+	  (SP_LGT(diag) != MAX(SP_NROWS(diag),SP_NCOLS(diag)) )) )
     PY_ERR_TYPE("invalid diag argument");
   
   if (Matrix_Check(diag)) {
@@ -892,6 +910,29 @@ spdiag(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	SP_VALZ(ret)[j] = MAT_BUFZ(diag)[j];
     }
     return (PyObject *)ret;
+  } 
+  else if (SpMatrix_Check(diag)) {
+
+    int k, id = MAX(DOUBLE, SP_ID(diag)), n = SP_LGT(diag);
+
+    spmatrix *ret = SpMatrix_New(n, n, SP_NNZ(diag), id);
+    if (!ret) return PyErr_NoMemory();
+    SP_COL(ret)[0] = 0;
+    
+    for (k=0; k<SP_NNZ(diag); k++) {
+      
+      SP_COL(ret)[SP_ROW(diag)[k]+1] = 1;
+      SP_ROW(ret)[k] = SP_ROW(diag)[k];
+      if (SP_ID(diag) == DOUBLE)
+	SP_VALD(ret)[k] = SP_VALD(diag)[k];
+      else
+	SP_VALZ(ret)[k] = SP_VALZ(diag)[k];	
+    }
+
+    for (k=0; k<n; k++) SP_COL(ret)[k+1] += SP_COL(ret)[k];
+      
+    return (PyObject *)ret;
+
   }
 
   int j, k, l, idx, id=DOUBLE, n=0, nnz=0;
@@ -975,13 +1016,666 @@ spdiag(PyTypeObject *type, PyObject *args, PyObject *kwds)
   return (PyObject *)ret;
 }
 
+matrix * dense(spmatrix *self);
+
+PyObject * matrix_elem_max(PyObject *self, PyObject *args, PyObject *kwrds)
+{
+  PyObject *A, *B;
+  if (!PyArg_ParseTuple(args, "OO:emax", &A, &B)) return NULL;
+
+  if (!(X_Matrix_Check(A) || PyNumber_Check(A)) ||
+      !(X_Matrix_Check(B) || PyNumber_Check(B))) 
+    PY_ERR_TYPE("arguments must be either matrices or python numbers");
+  
+  if (PyComplex_Check(A) || (X_Matrix_Check(A) && X_ID(A)==COMPLEX))
+    PY_ERR_TYPE("ordering not defined for complex numbers");
+
+  if (PyComplex_Check(B) || (X_Matrix_Check(B) && X_ID(B)==COMPLEX)) 
+    PY_ERR_TYPE("ordering not defined for complex numbers");
+ 
+  int a_is_number = PyNumber_Check(A) || 
+    (Matrix_Check(A) && MAT_LGT(A) == 1) || 
+    (SpMatrix_Check(A) && SP_LGT(A) == 1);
+  int b_is_number = PyNumber_Check(B) || 
+    (Matrix_Check(B) && MAT_LGT(B) == 1) ||
+    (SpMatrix_Check(B) && SP_LGT(B) == 1);
+  
+  int ida = PyNumber_Check(A) ? PyFloat_Check(A) : X_ID(A);
+  int idb = PyNumber_Check(B) ? PyFloat_Check(B) : X_ID(B);
+  int id  = MAX( ida, idb );
+
+  number a, b;
+  if (a_is_number) {
+    if (PyNumber_Check(A) || Matrix_Check(A))
+      convert_num[id](&a, A, PyNumber_Check(A), 0);
+    else 
+      a.d = (SP_LGT(A) ? SP_VALD(A)[0] : 0.0);
+  }
+  if (b_is_number) {
+    if (PyNumber_Check(B) || Matrix_Check(B))
+      convert_num[id](&b, B, PyNumber_Check(B), 0);
+    else 
+      b.d = (SP_LGT(B) ? SP_VALD(B)[0] : 0.0);
+  }
+  
+  if ((a_is_number && b_is_number) &&
+      (!X_Matrix_Check(A) && !X_Matrix_Check(B))) {
+    if (id == INT) 
+      return Py_BuildValue("i", MAX(a.i, b.i) );
+    else
+      return Py_BuildValue("d", MAX(a.d, b.d) );
+  }
+
+  if (!(a_is_number || b_is_number)) {
+    if (X_NROWS(A) != X_NROWS(B) || X_NCOLS(A) != X_NCOLS(B))
+      PY_ERR_TYPE("incompatible dimensions");
+  }
+
+  int m = ( !a_is_number ? X_NROWS(A) : (!b_is_number ? X_NROWS(B) : 1));
+  int n = ( !a_is_number ? X_NCOLS(A) : (!b_is_number ? X_NCOLS(B) : 1));
+  
+  if ((Matrix_Check(A) || a_is_number) || (Matrix_Check(B) || b_is_number)) {
+
+    int freeA = SpMatrix_Check(A) && (SP_LGT(A) > 1);
+    int freeB = SpMatrix_Check(B) && (SP_LGT(B) > 1);
+    if (freeA) { 
+      if (!(A = (PyObject *)dense((spmatrix *)A)) ) return PyErr_NoMemory();
+    }
+    if (freeB) { 
+      if (!(B = (PyObject *)dense((spmatrix *)B)) ) return PyErr_NoMemory();
+    }
+
+    PyObject *ret = (PyObject *)Matrix_New(m, n, id);
+    if (!ret) {
+      if (freeA) { Py_DECREF(A); }
+      if (freeB) { Py_DECREF(B); }
+      return PyErr_NoMemory();
+    }
+    int i;
+    for (i=0; i<m*n; i++) {
+      if (!a_is_number) convert_num[id](&a, A, 0, i);
+      if (!b_is_number) convert_num[id](&b, B, 0, i);
+      
+      if (id == INT)
+	MAT_BUFI(ret)[i] = MAX(a.i, b.i);
+      else
+	MAT_BUFD(ret)[i] = MAX(a.d, b.d);
+    }
+
+    if (freeA) { Py_DECREF(A); }
+    if (freeB) { Py_DECREF(B); }    
+    return ret;
+
+  } else {
+    
+    spmatrix *ret = SpMatrix_New(m, n, 0, DOUBLE);
+    if (!ret) return PyErr_NoMemory();
+
+    int j, ka = 0, kb = 0, kret = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  if (SP_VALD(A)[ka++] > 0.0) SP_COL(ret)[j+1]++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  if (SP_VALD(B)[kb++] > 0.0) SP_COL(ret)[j+1]++;
+	}
+	else {
+	  SP_COL(ret)[j+1]++; ka++; kb++;
+	}
+      }
+
+      while (ka < SP_COL(A)[j+1]) {
+	if (SP_VALD(A)[ka++] > 0.0) SP_COL(ret)[j+1]++;       
+      }
+
+      while (kb < SP_COL(B)[j+1]) {
+	if (SP_VALD(B)[kb++] > 0.0) SP_COL(ret)[j+1]++;       
+      }
+	
+    }
+
+    for (j=0; j<n; j++) SP_COL(ret)[j+1] += SP_COL(ret)[j];
+
+    int_t *newrow = malloc( sizeof(int_t)*SP_COL(ret)[n] );
+    double *newval = malloc( sizeof(double)*SP_COL(ret)[n] );
+    if (!newrow || !newval) {
+      free(newrow); free(newval); Py_DECREF(ret);
+      return PyErr_NoMemory();
+    }
+    free( ret->obj->rowind );
+    free( ret->obj->values );
+    ret->obj->rowind = newrow;
+    ret->obj->values = newval;
+
+    ka = 0; kb = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  if (SP_VALD(A)[ka] > 0.0) { 
+	    SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	    SP_VALD(ret)[kret++] = SP_VALD(A)[ka];
+	  }
+	  ka++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  if (SP_VALD(B)[kb] > 0.0) {
+	    SP_ROW(ret)[kret] = SP_ROW(B)[kb];
+	    SP_VALD(ret)[kret++] = SP_VALD(B)[kb];
+	  }
+	  kb++;
+	}
+	else {
+	  SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	  SP_VALD(ret)[kret] = MAX(SP_VALD(A)[ka], SP_VALD(B)[kb]);
+	  kret++; ka++; kb++;
+	}
+      }
+
+      while (ka < SP_COL(A)[j+1]) {
+	if (SP_VALD(A)[ka] > 0.0) {
+	    SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	    SP_VALD(ret)[kret++] = SP_VALD(A)[ka];
+	}
+	ka++;
+      }
+
+      while (kb < SP_COL(B)[j+1]) {
+	if (SP_VALD(B)[kb] > 0.0) {
+	  SP_ROW(ret)[kret] = SP_ROW(B)[kb];
+	  SP_VALD(ret)[kret++] = SP_VALD(B)[kb];
+	}
+	kb++;
+      }	
+    }
+
+    return (PyObject *)ret;
+  }
+}
+
+PyObject * matrix_elem_min(PyObject *self, PyObject *args, PyObject *kwrds)
+{
+  PyObject *A, *B;
+  if (!PyArg_ParseTuple(args, "OO:emin", &A, &B)) return NULL;
+
+  if (!(X_Matrix_Check(A) || PyNumber_Check(A)) ||
+      !(X_Matrix_Check(B) || PyNumber_Check(B))) 
+    PY_ERR_TYPE("arguments must be either matrices or python numbers");
+  
+  if (PyComplex_Check(A) || (X_Matrix_Check(A) && X_ID(A)==COMPLEX))
+    PY_ERR_TYPE("ordering not defined for complex numbers");
+
+  if (PyComplex_Check(B) || (X_Matrix_Check(B) && X_ID(B)==COMPLEX)) 
+    PY_ERR_TYPE("ordering not defined for complex numbers");
+ 
+  int a_is_number = PyNumber_Check(A) || 
+    (Matrix_Check(A) && MAT_LGT(A) == 1) || 
+    (SpMatrix_Check(A) && SP_LGT(A) == 1);
+  int b_is_number = PyNumber_Check(B) || 
+    (Matrix_Check(B) && MAT_LGT(B) == 1) ||
+    (SpMatrix_Check(B) && SP_LGT(B) == 1);
+  
+  int ida = PyNumber_Check(A) ? PyFloat_Check(A) : X_ID(A);
+  int idb = PyNumber_Check(B) ? PyFloat_Check(B) : X_ID(B);
+  int id  = MAX( ida, idb );
+
+  number a, b;
+  if (a_is_number) {
+    if (PyNumber_Check(A) || Matrix_Check(A))
+      convert_num[id](&a, A, PyNumber_Check(A), 0);
+    else 
+      a.d = (SP_LGT(A) ? SP_VALD(A)[0] : 0.0);
+  }
+  if (b_is_number) {
+    if (PyNumber_Check(B) || Matrix_Check(B))
+      convert_num[id](&b, B, PyNumber_Check(B), 0);
+    else 
+      b.d = (SP_LGT(B) ? SP_VALD(B)[0] : 0.0);
+  }
+
+  if ((a_is_number && b_is_number) &&
+      (!X_Matrix_Check(A) && !X_Matrix_Check(B))) {
+    if (id == INT) 
+      return Py_BuildValue("i", MIN(a.i, b.i) );
+    else
+      return Py_BuildValue("d", MIN(a.d, b.d) );
+  }
+
+  if (!(a_is_number || b_is_number)) {
+    if (X_NROWS(A) != X_NROWS(B) || X_NCOLS(A) != X_NCOLS(B))
+      PY_ERR_TYPE("incompatible dimensions");
+  }
+
+  int m = ( !a_is_number ? X_NROWS(A) : (!b_is_number ? X_NROWS(B) : 1));
+  int n = ( !a_is_number ? X_NCOLS(A) : (!b_is_number ? X_NCOLS(B) : 1));
+  
+  if ((Matrix_Check(A) || a_is_number) || (Matrix_Check(B) || b_is_number)) {
+
+    int freeA = SpMatrix_Check(A) && (SP_LGT(A) > 1);
+    int freeB = SpMatrix_Check(B) && (SP_LGT(B) > 1);
+    if (freeA) { 
+      if (!(A = (PyObject *)dense((spmatrix *)A)) ) return PyErr_NoMemory();
+    }
+    if (freeB) { 
+      if (!(B = (PyObject *)dense((spmatrix *)B)) ) return PyErr_NoMemory();
+    }
+
+    PyObject *ret = (PyObject *)Matrix_New(m, n, id);
+    if (!ret) {
+      if (freeA) { Py_DECREF(A); }
+      if (freeB) { Py_DECREF(B); }
+      return PyErr_NoMemory();
+    }
+    int i;
+    for (i=0; i<m*n; i++) {
+      if (!a_is_number) convert_num[id](&a, A, 0, i);
+      if (!b_is_number) convert_num[id](&b, B, 0, i);
+      
+      if (id == INT)
+	MAT_BUFI(ret)[i] = MIN(a.i, b.i);
+      else
+	MAT_BUFD(ret)[i] = MIN(a.d, b.d);
+    }
+
+    if (freeA) { Py_DECREF(A); }
+    if (freeB) { Py_DECREF(B); }    
+    return ret;
+
+  } else {
+    
+    spmatrix *ret = SpMatrix_New(m, n, 0, DOUBLE);
+    if (!ret) return PyErr_NoMemory();
+
+    int j, ka = 0, kb = 0, kret = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  if (SP_VALD(A)[ka++] < 0.0) SP_COL(ret)[j+1]++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  if (SP_VALD(B)[kb++] < 0.0) SP_COL(ret)[j+1]++;
+	}
+	else {
+	  SP_COL(ret)[j+1]++; ka++; kb++;
+	}
+      }
+
+      while (ka < SP_COL(A)[j+1]) {
+	if (SP_VALD(A)[ka++] < 0.0) SP_COL(ret)[j+1]++;       
+      }
+
+      while (kb < SP_COL(B)[j+1]) {
+	if (SP_VALD(B)[kb++] < 0.0) SP_COL(ret)[j+1]++;       
+      }
+	
+    }
+
+    for (j=0; j<n; j++) SP_COL(ret)[j+1] += SP_COL(ret)[j];
+
+    int_t *newrow = malloc( sizeof(int_t)*SP_COL(ret)[n] );
+    double *newval = malloc( sizeof(double)*SP_COL(ret)[n] );
+    if (!newrow || !newval) {
+      free(newrow); free(newval); Py_DECREF(ret);
+      return PyErr_NoMemory();
+    }
+    free( ret->obj->rowind );
+    free( ret->obj->values );
+    ret->obj->rowind = newrow;
+    ret->obj->values = newval;
+
+    ka = 0; kb = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  if (SP_VALD(A)[ka] < 0.0) { 
+	    SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	    SP_VALD(ret)[kret++] = SP_VALD(A)[ka];
+	  }
+	  ka++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  if (SP_VALD(B)[kb] < 0.0) {
+	    SP_ROW(ret)[kret] = SP_ROW(B)[kb];
+	    SP_VALD(ret)[kret++] = SP_VALD(B)[kb];
+	  }
+	  kb++;
+	}
+	else {
+	  SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	  SP_VALD(ret)[kret] = MIN(SP_VALD(A)[ka], SP_VALD(B)[kb]);
+	  kret++; ka++; kb++;
+	}
+      }
+
+      while (ka < SP_COL(A)[j+1]) {
+	if (SP_VALD(A)[ka] < 0.0) {
+	    SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	    SP_VALD(ret)[kret++] = SP_VALD(A)[ka];
+	}
+	ka++;
+      }
+
+      while (kb < SP_COL(B)[j+1]) {
+	if (SP_VALD(B)[kb] < 0.0) {
+	  SP_ROW(ret)[kret] = SP_ROW(B)[kb];
+	  SP_VALD(ret)[kret++] = SP_VALD(B)[kb];
+	}
+	kb++;
+      }	
+    }
+
+    return (PyObject *)ret;
+  }
+}
+
+PyObject * matrix_elem_mul(matrix *self, PyObject *args, PyObject *kwrds)
+{
+  PyObject *A, *B;
+  if (!PyArg_ParseTuple(args, "OO:emul", &A, &B)) return NULL;
+
+  if (!(X_Matrix_Check(A) || PyNumber_Check(A)) ||
+      !(X_Matrix_Check(B) || PyNumber_Check(B))) 
+    PY_ERR_TYPE("arguments must be either matrices or python numbers");
+   
+  int a_is_number = PyNumber_Check(A) || (Matrix_Check(A) && MAT_LGT(A) == 1);
+  int b_is_number = PyNumber_Check(B) || (Matrix_Check(B) && MAT_LGT(B) == 1);
+  
+  int ida, idb;
+  if (PyInt_Check(A)) { ida = INT; }
+  else if (PyFloat_Check(A)) { ida = DOUBLE; }
+  else if (PyComplex_Check(A)) { ida = COMPLEX; }
+  else { ida = X_ID(A); }
+
+  if (PyInt_Check(B)) { idb = INT; }
+  else if (PyFloat_Check(B)) { idb = DOUBLE; }
+  else if (PyComplex_Check(B)) { idb = COMPLEX; }
+  else { idb = X_ID(B); }
+
+  int id  = MAX( ida, idb );
+
+  number a, b;
+  if (a_is_number) convert_num[id](&a, A, PyNumber_Check(A), 0);
+  if (b_is_number) convert_num[id](&b, B, PyNumber_Check(B), 0);
+
+  if (a_is_number && b_is_number &&
+      (!X_Matrix_Check(A) && !X_Matrix_Check(B))) {
+    if (!X_Matrix_Check(A) && !X_Matrix_Check(B)) {
+      if (id == INT) 
+	return Py_BuildValue("i", a.i*b.i );
+      else if (id == DOUBLE)
+	return Py_BuildValue("d", a.d*b.d );
+      else {
+	number c;
+	c.z = a.z*b.z;
+	return znum2PyObject(&c, 0);
+      }
+    }
+  }
+
+  if (!(a_is_number || b_is_number)) {
+    if (X_NROWS(A) != X_NROWS(B) || X_NCOLS(A) != X_NCOLS(B))
+      PY_ERR_TYPE("incompatible dimensions");
+  }
+
+  int m = ( !a_is_number ? X_NROWS(A) : (!b_is_number ? X_NROWS(B) : 1));
+  int n = ( !a_is_number ? X_NCOLS(A) : (!b_is_number ? X_NCOLS(B) : 1));
+  
+  if ((Matrix_Check(A) || a_is_number) && (Matrix_Check(B) || b_is_number)) {
+
+    PyObject *ret = (PyObject *)Matrix_New(m, n, id);
+    if (!ret) return PyErr_NoMemory();
+
+    int i;
+    for (i=0; i<m*n; i++) {
+      if (!a_is_number) convert_num[id](&a, A, 0, i);
+      if (!b_is_number) convert_num[id](&b, B, 0, i);
+      
+      if (id == INT)
+	MAT_BUFI(ret)[i] = a.i*b.i;
+      else if (id == DOUBLE)
+	MAT_BUFD(ret)[i] = a.d*b.d;
+      else
+	MAT_BUFZ(ret)[i] = a.z*b.z;
+    }
+
+    return ret;
+
+  } 
+  else if (SpMatrix_Check(A) && !SpMatrix_Check(B)) {
+    
+    PyObject *ret = (PyObject *)SpMatrix_NewFromSpMatrix((spmatrix *)A, id);
+    if (!ret) return PyErr_NoMemory();
+
+    int j, k;
+    for (j=0; j<SP_NCOLS(A); j++) {
+      for (k=SP_COL(A)[j]; k<SP_COL(A)[j+1]; k++) {
+	if (!b_is_number) convert_num[id](&b, B, 0, j*m + SP_ROW(A)[k]);
+      
+	if (id == DOUBLE)
+	  SP_VALD(ret)[k] *= b.d;
+	else
+	  SP_VALZ(ret)[k] *= b.z;
+      }
+    }
+
+    return ret;
+  }
+  else if (SpMatrix_Check(B) && !SpMatrix_Check(A)) {
+    
+    PyObject *ret = (PyObject *)SpMatrix_NewFromSpMatrix((spmatrix *)B, id);
+    if (!ret) return PyErr_NoMemory();
+
+    int j, k;
+    for (j=0; j<SP_NCOLS(B); j++) {
+      for (k=SP_COL(B)[j]; k<SP_COL(B)[j+1]; k++) {
+	if (!a_is_number) convert_num[id](&a, A, 0, j*m + SP_ROW(B)[k]);
+      
+	if (id == DOUBLE)
+	  SP_VALD(ret)[k] *= a.d;
+	else
+	  SP_VALZ(ret)[k] *= a.z;
+      }
+    }
+
+    return ret;
+  }
+
+  else {
+    
+    spmatrix *ret = SpMatrix_New(m, n, 0, id);
+    if (!ret) return PyErr_NoMemory();
+
+    int j, ka = 0, kb = 0, kret = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  ka++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  kb++;
+	}
+	else {
+	  SP_COL(ret)[j+1]++; ka++; kb++;
+	}
+      }
+      
+      ka = SP_COL(A)[j+1];
+      kb = SP_COL(B)[j+1];	
+    }
+
+    for (j=0; j<n; j++) SP_COL(ret)[j+1] += SP_COL(ret)[j];
+
+    int_t *newrow = malloc( sizeof(int_t)*SP_COL(ret)[n] );
+    double *newval = malloc( E_SIZE[id]*SP_COL(ret)[n] );
+    if (!newrow || !newval) {
+      free(newrow); free(newval); Py_DECREF(ret);
+      return PyErr_NoMemory();
+    }
+    free( ret->obj->rowind );
+    free( ret->obj->values );
+    ret->obj->rowind = newrow;
+    ret->obj->values = newval;
+
+    ka = 0; kb = 0;
+    for (j=0; j<n; j++) {
+      
+      while (ka < SP_COL(A)[j+1] && kb < SP_COL(B)[j+1]) {
+	
+	if (SP_ROW(A)[ka] < SP_ROW(B)[kb]) {
+	  ka++;
+	}
+	else if (SP_ROW(A)[ka] > SP_ROW(B)[kb]) {
+	  kb++;
+	}
+	else {
+	  SP_ROW(ret)[kret] = SP_ROW(A)[ka];
+	  if (id == DOUBLE)
+	    SP_VALD(ret)[kret] = SP_VALD(A)[ka]*SP_VALD(B)[kb];
+	  else
+	    SP_VALZ(ret)[kret] = 
+	      (X_ID(A) == DOUBLE ? SP_VALD(A)[ka] : SP_VALZ(A)[ka])*
+	      (X_ID(B) == DOUBLE ? SP_VALD(B)[kb] : SP_VALZ(B)[kb]);
+	    
+	  kret++; ka++; kb++;
+	}
+      }
+
+      ka = SP_COL(A)[j+1];
+      kb = SP_COL(B)[j+1];	
+    }
+    
+    return (PyObject *)ret;
+  }
+}
+
+PyObject * matrix_elem_div(matrix *self, PyObject *args, PyObject *kwrds)
+{
+  PyObject *A, *B, *ret;
+  if (!PyArg_ParseTuple(args, "OO:ediv", &A, &B)) return NULL;
+
+  if (!(X_Matrix_Check(A) || PyNumber_Check(A)) ||
+      !(X_Matrix_Check(B) || PyNumber_Check(B))) 
+    PY_ERR_TYPE("arguments must be either matrices or python numbers");
+  
+  if (SpMatrix_Check(B))
+    PY_ERR_TYPE("elementwise division with sparse matrix\n");
+
+  int a_is_number = PyNumber_Check(A) || (Matrix_Check(A) && MAT_LGT(A) == 1);
+  int b_is_number = PyNumber_Check(B) || (Matrix_Check(B) && MAT_LGT(B) == 1);
+    
+  int ida, idb;
+  if (PyInt_Check(A)) { ida = INT; }
+  else if (PyFloat_Check(A)) { ida = DOUBLE; }
+  else if (PyComplex_Check(A)) { ida = COMPLEX; }
+  else { ida = X_ID(A); }
+
+  if (PyInt_Check(B)) { idb = INT; }
+  else if (PyFloat_Check(B)) { idb = DOUBLE; }
+  else if (PyComplex_Check(B)) { idb = COMPLEX; }
+  else { idb = X_ID(B); }
+
+  int id  = MAX( ida, idb );
+
+  number a, b;
+  if (a_is_number) convert_num[id](&a, A, PyNumber_Check(A), 0);
+  if (b_is_number) convert_num[id](&b, B, PyNumber_Check(B), 0);
+
+  if ((a_is_number && b_is_number) && 
+      (!X_Matrix_Check(A) && !Matrix_Check(B))) {
+    if (id == INT) {
+      if (b.i == 0) PY_ERR(PyExc_ArithmeticError, "division by zero");
+      return Py_BuildValue("i", a.i/b.i );
+    }
+    else if (id == DOUBLE) {
+      if (b.d == 0.0) PY_ERR(PyExc_ArithmeticError, "division by zero");
+      return Py_BuildValue("d", a.d/b.d );
+    }
+    else {
+      if (b.z == 0.0) PY_ERR(PyExc_ArithmeticError, "division by zero");
+      number c;
+      c.z = a.z/b.z;
+      return znum2PyObject(&c, 0);
+    }
+  }
+
+  if (!(a_is_number || b_is_number)) {
+    if (X_NROWS(A) != MAT_NROWS(B) || X_NCOLS(A) != MAT_NCOLS(B))
+      PY_ERR_TYPE("incompatible dimensions");
+  }
+
+  int m = ( !a_is_number ? X_NROWS(A) : (!b_is_number ? X_NROWS(B) : 1));
+  int n = ( !a_is_number ? X_NCOLS(A) : (!b_is_number ? X_NCOLS(B) : 1));
+  
+  if ((Matrix_Check(A) || a_is_number) && (Matrix_Check(B) || b_is_number)) {
+
+    if (!(ret = (PyObject *)Matrix_New(m, n, id)))
+      return PyErr_NoMemory();
+
+    int i;
+    for (i=0; i<m*n; i++) {
+      if (!a_is_number) convert_num[id](&a, A, 0, i);
+      if (!b_is_number) convert_num[id](&b, B, 0, i);
+      
+      if (id == INT) {
+	if (b.i == 0) goto divzero;
+	MAT_BUFI(ret)[i] = a.i/b.i;
+      }
+      else if (id == DOUBLE) {
+	if (b.d == 0) goto divzero;
+	MAT_BUFD(ret)[i] = a.d/b.d;
+      }
+      else {
+	if (b.z == 0) goto divzero;
+	MAT_BUFZ(ret)[i] = a.z/b.z;
+      }
+    }
+
+    return ret;
+  } 
+  else { // (SpMatrix_Check(A) && !SpMatrix_Check(B)) {
+    
+    if (!(ret = (PyObject *)SpMatrix_NewFromSpMatrix((spmatrix *)A, id)))
+      return PyErr_NoMemory();
+
+    int j, k;
+    for (j=0; j<SP_NCOLS(A); j++) {
+      for (k=SP_COL(A)[j]; k<SP_COL(A)[j+1]; k++) {
+	if (!b_is_number) convert_num[id](&b, B, 0, j*m + SP_ROW(A)[k]);
+      
+	if (id == DOUBLE) {
+	  if (b.d == 0.0) goto divzero;
+	  SP_VALD(ret)[k] /= b.d;
+	}
+	else {
+	  if (b.z == 0.0) goto divzero;
+	  SP_VALZ(ret)[k] /= b.z;
+	}
+      }
+    }
+
+    return ret;
+  }
+
+ divzero:
+  Py_DECREF(ret);
+  PY_ERR(PyExc_ArithmeticError, "division by zero");
+}
+
 extern PyObject * matrix_exp(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_log(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_sqrt(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_cos(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_sin(matrix *, PyObject *, PyObject *) ;
-extern PyObject * matrix_elem_mul(matrix *, PyObject *, PyObject *) ;
-extern PyObject * matrix_elem_div(matrix *, PyObject *, PyObject *) ;
 
 static PyMethodDef base_functions[] = {
   {"exp", (PyCFunction)matrix_exp, METH_VARARGS|METH_KEYWORDS, 
@@ -999,12 +1693,15 @@ static PyMethodDef base_functions[] = {
   {"gemv", (PyCFunction)base_gemv, METH_VARARGS|METH_KEYWORDS, doc_gemv},
   {"syrk", (PyCFunction)base_syrk, METH_VARARGS|METH_KEYWORDS, doc_syrk},
   {"symv", (PyCFunction)base_symv, METH_VARARGS|METH_KEYWORDS, doc_symv},
-  {"mul", (PyCFunction)matrix_elem_mul, METH_VARARGS|METH_KEYWORDS, 
+  {"emul", (PyCFunction)matrix_elem_mul, METH_VARARGS|METH_KEYWORDS, 
    "elementwise product of two matrices"},
-  {"div", (PyCFunction)matrix_elem_div, METH_VARARGS|METH_KEYWORDS, 
+  {"ediv", (PyCFunction)matrix_elem_div, METH_VARARGS|METH_KEYWORDS, 
    "elementwise division between two matrices"},
-  {"sparse", (PyCFunction)sparse, METH_VARARGS|METH_KEYWORDS, 
-   "convenience function for creating sparse matrices"},
+  {"emin", (PyCFunction)matrix_elem_min, METH_VARARGS|METH_KEYWORDS, 
+   "elementwise minimum between two matrices"},
+  {"emax", (PyCFunction)matrix_elem_max, METH_VARARGS|METH_KEYWORDS, 
+   "elementwise maximum between two matrices"},
+  {"sparse", (PyCFunction)sparse, METH_VARARGS|METH_KEYWORDS, doc_sparse},
   {"spdiag", (PyCFunction)spdiag, METH_VARARGS|METH_KEYWORDS, doc_spdiag},
   {NULL}		/* sentinel */
 };
@@ -1013,7 +1710,7 @@ PyMODINIT_FUNC
 initbase(void)
 {  
   static void *base_API[8];
-  PyObject *c_api_object;
+  PyObject *base_mod, *c_api_object;
   
   if (!(base_mod = Py_InitModule3("base", base_functions, base__doc__)))
     return;
