@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 1.0.
+ * This file is part of CVXOPT version 1.1.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@ typedef struct {
 static const char PY_ARRAY_TC[3] = { 'i', 'f', 'c' }; 
 
 /* prototyping and forward declarations */
-extern PyObject *base_mod;
 extern void (*axpy[])(int *, number *, void *, int *, void *, int *) ;
 extern void (*scal[])(int *, number *, void *, int *) ;
 extern void (*gemm[])(char *, char *, int *, int *, int *, void *, void *, 
@@ -524,14 +523,111 @@ matrix_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
   return (PyObject *)ret;
 }
 
+PyObject *matrix_sub(PyObject *, PyObject *);
+
+static PyObject * 
+matrix_richcompare(PyObject *self, PyObject *other, int op) {
+
+  PY_ERR(PyExc_NotImplementedError, "matrix comparison not implemented");  
+}
+
+/*
+static PyObject * 
+matrix_richcompare(PyObject *self, PyObject *other, int op) {
+
+  if (((Matrix_Check(self) || SpMatrix_Check(self)) && X_ID(self)==COMPLEX) ||
+      ((Matrix_Check(other) || SpMatrix_Check(other)) && X_ID(other)==COMPLEX))
+    PY_ERR_TYPE("complex comparison not defined");
+    
+  if (PyComplex_Check(self) || PyComplex_Check(other))
+    PY_ERR_TYPE("complex comparison not defined");
+    
+  if (!Matrix_Check(self) && !SpMatrix_Check(self) && !PyNumber_Check(self)) 
+  {
+    Py_INCREF(Py_NotImplemented);
+    return Py_NotImplemented;
+  }
+
+  if (!Matrix_Check(other) && !SpMatrix_Check(other) && !PyNumber_Check(other))
+  {
+    Py_INCREF(Py_NotImplemented);
+    return Py_NotImplemented;
+  }
+      
+  if ( (Matrix_Check(self) || SpMatrix_Check(other)) &&
+      (Matrix_Check(other) || SpMatrix_Check(other)) && 
+      ((X_NROWS(self)*X_NCOLS(self) > 1) &&
+	  (X_NROWS(other)*X_NCOLS(other) > 1)) &&
+      (X_NROWS(self) != X_NROWS(other) || X_NCOLS(self) != X_NCOLS(other)) )
+    PY_ERR_TYPE("matrices in comparison must have same dimensions");
+
+  PyObject *tmp = matrix_sub(self, other);
+  if (!tmp) return PyErr_NoMemory();
+
+  PyObject *ret = (PyObject *)Matrix_New(X_NROWS(self), X_NCOLS(self), INT);
+  if (!ret) {
+    Py_DECREF(tmp); 
+    return PyErr_NoMemory();
+  }
+
+  int i;
+  switch (op) {
+  case Py_LT :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] < 0:
+	  MAT_BUFD(tmp)[i] < 0);
+    break;
+    
+  case Py_LE :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] <= 0:
+	  MAT_BUFD(tmp)[i] <= 0);
+    break;
+    
+  case Py_GT :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] > 0:
+	  MAT_BUFD(tmp)[i] > 0);
+    break;
+      
+  case Py_GE :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] >= 0:
+	  MAT_BUFD(tmp)[i] >= 0);
+    break;
+
+  case Py_EQ :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] == 0:
+	  MAT_BUFD(tmp)[i] == 0);
+    break;
+    
+  case Py_NE :
+    for (i=0; i<MAT_LGT(ret); i++) 
+      MAT_BUFI(ret)[i] = (MAT_ID(tmp) == INT ? MAT_BUFI(tmp)[i] != 0:
+	  MAT_BUFD(tmp)[i] != 0);
+    break;
+      
+  default: break;
+  }
+  
+  Py_DECREF(tmp);
+  return ret;
+}
+*/
+
 static PyObject *
 matrix_str(matrix *self) {
 
+  PyObject *cvxopt = PyImport_ImportModule("cvxopt");
   PyObject *str, *ret;
-   
-  if (!(str = PyObject_GetAttrString(base_mod, "matrix_str")))
-    PY_ERR(PyExc_KeyError, "missing 'matrix_str' in 'base_options'");
+  
+  if (!(str = PyObject_GetAttrString(cvxopt, "matrix_str"))) {
+    Py_DECREF(cvxopt);
+    PY_ERR(PyExc_KeyError, "missing 'matrix_str' in 'cvxopt'");
+  }
 
+  Py_DECREF(cvxopt);
   if (!PyCallable_Check(str)) PY_ERR_TYPE("'matrix_str' is not callable");
 
   ret = PyObject_CallFunctionObjArgs(str, (PyObject *)self, NULL);
@@ -543,17 +639,21 @@ matrix_str(matrix *self) {
 static PyObject *
 matrix_repr(matrix *self) {
 
+  PyObject *cvxopt = PyImport_ImportModule("cvxopt");
   PyObject *repr, *ret;
-   
-  if (!(repr = PyObject_GetAttrString(base_mod, "matrix_repr")))
-    PY_ERR(PyExc_KeyError, "missing 'matrix_repr' in 'base_options'");
+  
+  if (!(repr = PyObject_GetAttrString(cvxopt, "matrix_repr"))) {
+    Py_DECREF(cvxopt);
+    PY_ERR(PyExc_KeyError, "missing 'matrix_repr' in 'cvxopt'");
+  }
 
+  Py_DECREF(cvxopt);
   if (!PyCallable_Check(repr)) PY_ERR_TYPE("'matrix_repr' is not callable");
 
   ret = PyObject_CallFunctionObjArgs(repr, (PyObject *)self, NULL);
   Py_DECREF(repr);
 
-  return ret;
+  return ret;  
 }
 
 /*
@@ -1182,7 +1282,7 @@ PyTypeObject matrix_tp = {
 	0,					/* tp_doc */
 	0,					/* tp_traverse */
 	0,					/* tp_clear */
-	0,	      		                /* tp_richcompare */
+	(richcmpfunc)matrix_richcompare,        /* tp_richcompare */
 	0,					/* tp_weaklistoffset */
 	(getiterfunc)matrix_iter,		/* tp_iter */
 	0,	       	                        /* tp_iternext */
@@ -1416,7 +1516,6 @@ static PyObject * matrix_isub(PyObject *self,PyObject *other)
 static PyObject * 
 matrix_mul_generic(PyObject *self, PyObject *other, int inplace) 
 {
-
   if (!(Matrix_Check(self) || PY_NUMBER(self)) || 
       !(Matrix_Check(other) || PY_NUMBER(other))) {
     Py_INCREF(Py_NotImplemented);
@@ -1680,6 +1779,17 @@ static PyObject * matrix_pow(PyObject *self, PyObject *other)
   return (PyObject *)Y;
 }
 
+static int matrix_nonzero(matrix *self)
+{
+  int i, res = 0;
+  for (i=0; i<MAT_LGT(self); i++) {
+    if ((MAT_ID(self) == INT) && (MAT_BUFI(self)[i] != 0)) res = 1;
+    else if ((MAT_ID(self) == DOUBLE) && (MAT_BUFD(self)[i] != 0)) res = 1;    
+    else if ((MAT_ID(self) == COMPLEX) && (MAT_BUFZ(self)[i] != 0)) res = 1;
+  }
+  
+  return res;
+}
 
 static PyNumberMethods matrix_as_number = {
 	(binaryfunc)matrix_add,	/*nb_add*/
@@ -1692,7 +1802,7 @@ static PyNumberMethods matrix_as_number = {
 	(unaryfunc)matrix_neg,	/*nb_negative*/
 	(unaryfunc)matrix_pos,	/*nb_positive*/
 	(unaryfunc)matrix_abs,	/*nb_absolute*/
-	0,	                /*nb_nonzero*/
+	(inquiry)matrix_nonzero,/*nb_nonzero*/
 	0,	                /*nb_invert*/
 	0,	                /*nb_lshift*/
 	0,	                /*nb_rshift*/
@@ -2050,67 +2160,4 @@ PyObject * matrix_sin(matrix *self, PyObject *args, PyObject *kwrds)
   else PY_ERR_TYPE("argument must a be a number or dense matrix");
 }
 
-PyObject * matrix_elem_mul(matrix *self, PyObject *args, PyObject *kwrds)
-{
-  matrix *A, *B;
-
-  if (!PyArg_ParseTuple(args, "OO", &A, &B)) return NULL;
-
-  if (!Matrix_Check(A) || !Matrix_Check(B) || MAT_ID(A) != MAT_ID(B))
-    PY_ERR_TYPE("arguments must be matrices of same type");
-
-  if (MAT_NROWS(A) != MAT_NROWS(B) || MAT_NCOLS(A) != MAT_NCOLS(B))
-    PY_ERR_TYPE("arguments must have same dimensions");
-
-  matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), MAT_ID(A));
-  if (!ret) return PyErr_NoMemory();
-
-  int i;
-  for (i=0; i<MAT_LGT(A); i++) {
-    switch (MAT_ID(A)) {
-    case INT: MAT_BUFI(ret)[i] = MAT_BUFI(A)[i]*MAT_BUFI(B)[i]; break;
-    case DOUBLE: MAT_BUFD(ret)[i] = MAT_BUFD(A)[i]*MAT_BUFD(B)[i]; break;
-    case COMPLEX: MAT_BUFZ(ret)[i] = MAT_BUFZ(A)[i]*MAT_BUFZ(B)[i]; break;    
-    }    
-  }
-  
-  return (PyObject *)ret;
-}
-
-PyObject * matrix_elem_div(matrix *self, PyObject *args, PyObject *kwrds)
-{
-  matrix *A, *B;
-
-  if (!PyArg_ParseTuple(args, "OO", &A, &B)) return NULL;
-
-  if (!Matrix_Check(A) || !Matrix_Check(B) || MAT_ID(A) != MAT_ID(B))
-    PY_ERR_TYPE("arguments must be matrices of same type");
-
-  if (MAT_NROWS(A) != MAT_NROWS(B) || MAT_NCOLS(A) != MAT_NCOLS(B))
-    PY_ERR_TYPE("arguments must have same dimensions");
-
-  matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), MAT_ID(A));
-  if (!ret) return PyErr_NoMemory();
-
-  int i;
-  for (i=0; i<MAT_LGT(A); i++) {
-    switch (MAT_ID(A)) {
-    case INT: 
-      if (MAT_BUFI(B)[i] == 0) goto divzero;
-      MAT_BUFI(ret)[i] = MAT_BUFI(A)[i]/MAT_BUFI(B)[i]; break;
-    case DOUBLE: 
-      if (MAT_BUFD(B)[i] == 0.0) goto divzero;
-      MAT_BUFD(ret)[i] = MAT_BUFD(A)[i]/MAT_BUFD(B)[i]; break;
-    case COMPLEX: 
-      if (MAT_BUFZ(B)[i] == 0.0) goto divzero;
-      MAT_BUFZ(ret)[i] = MAT_BUFZ(A)[i]/MAT_BUFZ(B)[i]; break;    
-    }    
-  }
-  
-  return (PyObject *)ret;
-
- divzero:
-  Py_DECREF(ret);
-  PY_ERR(PyExc_ArithmeticError, "division by zero");
-}
 
