@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 1.0.
+ * This file is part of CVXOPT version 1.1.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -360,7 +360,9 @@ static PyObject* solvesdp(PyObject *self, PyObject *args,
 	goto done;
     }
     DSDPStopReason(sdp, &info);
-    if (info != DSDP_CONVERGED){
+    if (info != DSDP_CONVERGED && info != DSDP_SMALL_STEPS && 
+        info != DSDP_INDEFINITE_SCHUR_MATRIX && info != DSDP_MAX_IT 
+        && info != DSDP_NUMERICAL_ERROR && info != DSDP_UPPERBOUND ){
         PyErr_SetObject(PyExc_ArithmeticError, Py_BuildValue("i",info));
         t = NULL;
         goto done;
@@ -376,23 +378,28 @@ static PyObject* solvesdp(PyObject *self, PyObject *args,
     }
     DSDPGetSolutionType(sdp, &status);
 
-    switch (status){
-        case DSDP_PDFEASIBLE:
-            PyTuple_SET_ITEM(t, 0, (PyObject *) 
-                PyString_FromString("DSDP_PDFEASIBLE"));
-	    break;
-        case DSDP_UNBOUNDED:
-            PyTuple_SET_ITEM(t, 0, (PyObject *) 
-                PyString_FromString("DSDP_UNBOUNDED"));
-	    break;
-        case DSDP_INFEASIBLE:
-            PyTuple_SET_ITEM(t, 0, (PyObject *) 
-                PyString_FromString("DSDP_INFEASIBLE"));
-	    break;
-        case DSDP_PDUNKNOWN:
-            PyTuple_SET_ITEM(t, 0, (PyObject *) 
-                PyString_FromString("DSDP_UNKNOWN"));
-	    break;
+    if (info == DSDP_CONVERGED) {
+        switch (status){
+            case DSDP_PDFEASIBLE:
+                PyTuple_SET_ITEM(t, 0, (PyObject *) 
+                    PyString_FromString("DSDP_PDFEASIBLE"));
+	        break;
+            case DSDP_UNBOUNDED:
+                PyTuple_SET_ITEM(t, 0, (PyObject *) 
+                    PyString_FromString("DSDP_UNBOUNDED"));
+	        break;
+            case DSDP_INFEASIBLE:
+                PyTuple_SET_ITEM(t, 0, (PyObject *) 
+                    PyString_FromString("DSDP_INFEASIBLE"));
+	        break;
+            case DSDP_PDUNKNOWN:
+                PyTuple_SET_ITEM(t, 0, (PyObject *) 
+                    PyString_FromString("DSDP_UNKNOWN"));
+	        break;
+        }
+    } else {
+        PyTuple_SET_ITEM(t, 0, (PyObject *) 
+        PyString_FromString("DSDP_UNKNOWN"));
     }
 
     DSDPGetY(sdp, MAT_BUFD(x), n);
