@@ -8,7 +8,7 @@ to the quadratic programming solver from MOSEK.
 
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 1.0.
+# This file is part of CVXOPT version 1.1.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -139,24 +139,89 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
 
     Output arguments.
 
-        cpl() returns a dictionary with keys 'status', 'x', 'snl', 'sl',
-        'znl', 'zl', 'y'.
+        Returns a dictionary with keys 'status', 'x', 'snl', 'sl', 'znl', 
+        'zl', 'y', 'primal objective', 'dual objective', 'gap', 
+        'relative gap', 'primal infeasibility', 'dual infeasibility',
+        'primal slack', 'dual slack'.
 
-        If status is 'optimal', x, snl, sl are approximate solutions of
-        the primal problem 
+        The 'status' field has values 'optimal' or 'unknown'.
+        If status is 'optimal', x, snl, sl, y, znl, zl are an approximate 
+        solution of the primal and dual optimality conditions 
 
-            minimize    c'*x 
-            subject to  f(x) + snl = 0
-                        G*x + sl  = h
-                        A*x =  b,
-                        snl >= 0, sl >= 0
+            f(x) + snl = 0,  G*x + sl = h,  A*x = b 
+            Df(x)'*znl + G'*zl + A'*y + c = 0 
+            snl >= 0,  znl >= 0,  sl >= 0,  zl >= 0
+            snl'*znl + sl'* zl = 0.
 
-        and y, znl, zl are approximate solutions of the dual problem
+        If status is 'unknown', x, snl, sl, y, znl, zl are the last
+        iterates before termination.  They satisfy snl > 0, znl > 0, 
+        sl > 0, zl > 0, but are not necessarily feasible.
 
-            maximize    inf_x (c'*x + znl'*f(x) + zl'*(G*x-h) + y'*(A*x-b))
-            subject to  znl >= 0, zl >= 0.
+        The values of the other fields are defined as follows.
 
-        If status is 'unknown', x, s, snl, sl, y, znl, zl are None.
+        - 'primal objective': the primal objective c'*x.
+
+        - 'dual objective': the dual objective 
+
+              L(x,y,znl,zl) = c'*x + znl'*f(x) + zl'*(G*x-h) + y'*(A*x-b).
+
+        - 'gap': the duality gap snl'*znl + sl'*zl.
+
+        - 'relative gap': the relative gap, defined as 
+
+              gap / -primal objective
+
+          if the primal objective is negative, 
+
+              gap / dual objective
+
+          if the dual objective is positive, and None otherwise.
+
+        - 'primal infeasibility': the residual in the primal constraints,
+          defined as 
+
+              || (f(x) + snl, G*x + sl - h, A*x-b) ||_2  
+
+          divided by 
+
+              max(1, || (f(x0) + 1, G*x0 + 1 - h, A*x0 - b) ||_2 )
+
+          where x0 is the point returned by F().
+
+        - 'dual infeasibility': the residual in the dual constraints,
+          defined as
+
+              || c + Df(x)'*znl + G'*zl + A'*y ||_2
+ 
+          divided by 
+
+              max(1, || c + Df(x0)'*1 + G'*1 ||_2 ).
+
+        - 'primal slack': the smallest primal slack, min( min_k sl_k,
+          sup {t | sl >= te} ) where 
+
+              e = ( e_0, e_1, ..., e_N, e_{N+1}, ..., e_{M+N} )
+    
+          is the identity vector in C.  e_0 is an ml-vector of ones, 
+          e_k, k = 1,..., N, is the unit vector (1,0,...,0) of length
+          mq[k], and e_k = vec(I) where I is the identity matrix of order
+          ms[k].
+
+        - 'dual slack': the smallest dual slack, min( min_k zl_k,
+          sup {t | zl >= te} ).
+               
+
+        If the exit status is 'optimal', then the primal and dual
+        infeasibilities are guaranteed to be less than 
+        solvers.options['feastol'] (default 1e-7).  The gap is less than
+        solvers.options['abstol'] (default 1e-7) or the relative gap is 
+        less than solvers.options['reltol'] (defaults 1e-6).     
+
+        Termination with status 'unknown' indicates that the algorithm 
+        failed to find a solution that satisfies the specified tolerances.
+        In some cases, the returned solution may be fairly accurate.  If
+        the primal and dual infeasibilities, the gap, and the relative gap
+        are small, then x, y, snl, sl, znl, zl are close to optimal.
 
 
     Advanced usage.
@@ -343,8 +408,14 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
     try: FEASTOL = options['feastol']
     except KeyError: FEASTOL = 1e-7
     else: 
-        if type(FEASTOL) is not float and type(FEASTOL) is not int: 
-            raise ValueError, "options['feastol'] must be a scalar"
+        if (type(FEASTOL) is not float and type(FEASTOL) is not int) or \
+            FEASTOL <= 0.0: 
+            raise ValueError, "options['feastol'] must be a positive "\
+                "scalar"
+
+    if RELTOL <= 0.0 and ABSTOL <= 0.0:
+        raise ValueError, "at least one of options['reltol'] and " \
+            "options['abstol'] must be positive"
 
     try: show_progress = options['show_progress']
     except KeyError: show_progress = True
@@ -552,7 +623,7 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
             "dres")
 
     relaxed_iters = 0
-    for iters in xrange(MAXITERS):  
+    for iters in xrange(MAXITERS + 1):  
 
         if refinement or DEBUG:  
             # We need H to compute residuals of KKT equations.
@@ -622,6 +693,8 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
         fG(x, rzl, beta = 1.0)
         reszl = misc.snrm2(rzl, dims)
 
+        # Statistics for stopping criteria.
+
         # pcost = c'*x
         # dcost = c'*x + y'*(A*x-b) + znl'*f(x) + zl'*(G*x-h)
         #       = c'*x + y'*(A*x-b) + znl'*(f(x)+snl) + zl'*(G*x-h+sl) 
@@ -630,7 +703,12 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
         pcost = xdot(c,x)
         dcost = pcost + ydot(y, ry) + blas.dot(z[:mnl], rznl) + \
             misc.sdot(z[mnl:], rzl, dims) - gap
-
+        if pcost < 0.0:
+            relgap = gap / -pcost
+        elif dcost > 0.0:
+            relgap = gap / dcost
+        else:
+            relgap = None
         pres = math.sqrt( resy**2 + resznl**2 + reszl**2 )
         dres = resx
         if iters == 0: 
@@ -642,24 +720,42 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
             theta1 = 1.0 / gap0
             theta2 = 1.0 / resx0
             theta3 = 1.0 / resznl0
-
         phi = theta1 * gap + theta2 * resx + theta3 * resznl
+        pres = pres / pres0
+        dres = dres / dres0
+
         if show_progress:
             print "%2d: % 8.4e % 8.4e % 4.0e% 7.0e% 7.0e" \
-                %(iters, pcost, dcost, gap, pres/pres0, dres/dres0) 
+                %(iters, pcost, dcost, gap, pres, dres) 
 
         # Stopping criteria.    
-        if pres/pres0 <= FEASTOL and dres/dres0 <= FEASTOL and \
-            ( gap <= ABSTOL or (dcost > 0 and gap/dcost <= RELTOL) or
-            (pcost < 0 and gap/(-pcost) <= RELTOL) ):
+        if ( pres <= FEASTOL and dres <= FEASTOL and ( gap <= ABSTOL or 
+            (relgap is not None and relgap <= RELTOL) )) or \
+            iters == MAXITERS:
             sl, zl = s[mnl:], z[mnl:]
             ind = dims['l'] + sum(dims['q'])
             for m in dims['s']:
                 misc.symm(sl, m, ind)
                 misc.symm(zl, m, ind)
                 ind += m**2
-            return {'status': 'optimal', 'x': x,  'y': y, 'znl': z[:mnl],  
-                'zl': zl, 'snl': s[:mnl], 'sl': sl}
+            ts = misc.max_step(s, dims, mnl)
+            tz = misc.max_step(z, dims, mnl)
+            if iters == MAXITERS:
+                if show_progress:
+                    print "Terminated (maximum number of iterations "\
+                        "reached)."
+                status = 'unknown'
+            else:
+                if show_progress:
+                    print "Optimal solution found."
+                status = 'optimal'
+
+            return {'status': status, 'x': x,  'y': y, 'znl': z[:mnl],  
+                'zl': zl, 'snl': s[:mnl], 'sl': sl, 'gap': gap, 
+                'relative gap': relgap, 'primal objective': pcost, 
+                'dual objective': dcost,  'primal slack': -ts, 
+                'dual slack': -tz, 'primal infeasibility': pres,
+                'dual infeasibility': dres }
 
 
         # Compute initial scaling W: 
@@ -684,6 +780,7 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
         
         try: f3 = kktsolver(x, z[:mnl], W)
         except ArithmeticError: 
+            singular_kkt_matrix = False
             if iters == 0:
                 raise ValueError, "Rank(A) < p or "\
                     "Rank([H(x); A; Df(x); G]) < n"
@@ -720,10 +817,30 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
 
                 try: f3 = kktsolver(x, z[:mnl], W)
                 except ArithmeticError: 
-                    raise ArithmeticError, "singular KKT matrix"
+                     singular_kkt_matrix = True
 
             else:  
-                raise ArithmeticError, "singular KKT matrix"
+                 singular_kkt_matrix = True
+
+            if singular_kkt_matrix:
+                sl, zl = s[mnl:], z[mnl:]
+                ind = dims['l'] + sum(dims['q'])
+                for m in dims['s']:
+                    misc.symm(sl, m, ind)
+                    misc.symm(zl, m, ind)
+                    ind += m**2
+                ts = misc.max_step(s, dims, mnl)
+                tz = misc.max_step(z, dims, mnl)
+                if show_progress:
+                    print "Terminated (singular KKT matrix)."
+                status = 'unknown'
+                return {'status': status, 'x': x,  'y': y, 
+                    'znl': z[:mnl],  'zl': zl, 'snl': s[:mnl], 
+                    'sl': sl, 'gap': gap, 'relative gap': relgap, 
+                    'primal objective': pcost, 'dual objective': dcost,  
+                    'primal infeasibility': pres, 
+                    'dual infeasibility': dres, 'primal slack': -ts,
+                    'dual slack': -tz }
 
 
         # f4_no_ir(x, y, z, s) solves
@@ -893,7 +1010,23 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
                     raise ValueError, "Rank(A) < p or "\
                         "Rank([H(x); A; Df(x); G]) < n"
                 else:
-                    raise ArithmeticError, "singular KKT matrix"
+                    sl, zl = s[mnl:], z[mnl:]
+                    ind = dims['l'] + sum(dims['q'])
+                    for m in dims['s']:
+                        misc.symm(sl, m, ind)
+                        misc.symm(zl, m, ind)
+                        ind += m**2
+                    ts = misc.max_step(s, dims, mnl)
+                    tz = misc.max_step(z, dims, mnl)
+                    if show_progress:
+                        print "Terminated (singular KKT matrix)."
+                    return {'status': 'unknown', 'x': x,  'y': y, 
+                        'znl': z[:mnl],  'zl': zl, 'snl': s[:mnl], 
+                        'sl': sl, 'gap': gap, 'relative gap': relgap, 
+                        'primal objective': pcost, 'dual objective': dcost,
+                        'primal infeasibility': pres, 
+                        'dual infeasibility': dres, 'primal slack': -ts,
+                        'dual slack': -tz }
 
             # Inner product ds'*dz and unscaled steps are needed in the 
             # line search.
@@ -1230,8 +1363,6 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
 
         gap = blas.dot(lmbda, lmbda) 
 
-    return {'status': 'unknown', 'x': None,  'y': None, 'znl': None, 
-        'zl': None, 'snl': None, 'sl': None}
 
 
 def cp(F, G = None, h = None, dims = None, A = None, b = None,
@@ -1344,23 +1475,37 @@ def cp(F, G = None, h = None, dims = None, A = None, b = None,
     Output arguments.
 
         cp() returns a dictionary with keys 'status', 'x', 'snl', 'sl',
-        'znl', 'zl', 'y'.
+        'znl', 'zl', 'y', 'primal objective', 'dual objective', 'gap',
+        'relative gap', 'primal infeasibility', 'dual infeasibility',
+        'primal slack', 'dual slack'.
 
-        If status is 'optimal', x, snl, sl are approximate solutions of
-        the primal problem 
+        The 'status' field has values 'optimal' or 'unknown'.
+        If status is 'optimal', x, snl, sl, y, znl, zl  are approximate 
+        solutions of the primal and dual optimality conditions
 
-            minimize    f0(x)  
-            subject to  fk(x) + snl_k = 0, k = 1,...,mnl
-                        G*x + sl  = h
-                        A*x =  b,
-                        snl >= 0, sl >= 0
+            f(x)[1:] + snl = 0,  G*x + sl = h,  A*x = b 
+            Df(x)'*[1; znl] + G'*zl + A'*y + c = 0 
+            snl >= 0,  znl >= 0,  sl >= 0,  zl >= 0
+            snl'*znl + sl'* zl = 0.
 
-        and y, znl, zl are approximate solutions of the dual problem
+        If status is 'unknown', x, snl, sl, y, znl, zl are the last
+        iterates before termination.  They satisfy snl > 0, znl > 0, 
+        sl > 0, zl > 0, but are not necessarily feasible.
 
-            maximize    inf_x ( (1,znl)'*f(x) + zl'*(G*x-h) + y'*(A*x-b) )
-            subject to  znl >= 0, zl >= 0.
+        The values of the other fields are the values returned by cpl()
+        applied to the epigraph form problem
 
-        If status is 'unknown' x, s, snl, sl, y, znl, zl are None.
+            minimize   t 
+            subjec to  f0(x) <= t
+                       fk(x) <= 0, k = 1, ..., mnl
+                       G*x <= h
+                       A*x = b.
+
+        Termination with status 'unknown' indicates that the algorithm 
+        failed to find a solution that satisfies the specified tolerances.
+        In some cases, the returned solution may be fairly accurate.  If
+        the primal and dual infeasibilities, the gap, and the relative gap
+        are small, then x, y, snl, sl, znl, zl are close to optimal.
 
 
     Advanced usage.
@@ -1500,6 +1645,7 @@ def cp(F, G = None, h = None, dims = None, A = None, b = None,
 
        options['show_progress'] True/False (default: True)
        options['maxiters'] positive integer (default: 100)
+       options['refinement'] nonnegative integer (default: 1)
        options['abstol'] scalar (default: 1e-7)
        options['reltol'] scalar (default: 1e-6)
        options['feastol'] scalar (default: 1e-7).
@@ -1609,7 +1755,9 @@ def cp(F, G = None, h = None, dims = None, A = None, b = None,
     #     subject to f0(x) - t <= 0
     #                f1(x) <= 0
     #                ...
-    #                fmnl(x) <= 0.
+    #                fmnl(x) <= 0
+    #                G*x <= h
+    #                A*x = b.
     #
     # The epigraph form variable is stored as a list [x, t].
 
@@ -1815,13 +1963,8 @@ def cp(F, G = None, h = None, dims = None, A = None, b = None,
     sol = cpl(c, F_e, G_e, h, dims, A_e, b, kktsolver_e, xnewcopy_e, 
          xdot_e, xaxpy_e, xscal_e)
 
-    if sol['status'] == 'optimal':
-        sol['x'] = sol['x'][0]
-        sol['znl'], sol['snl'] = sol['znl'][1:], sol['snl'][1:]
-    else:
-        sol['x'] = None
-        sol['zl'], sol['sl'] = None, None
-        sol['znl'], sol['snl'] = None, None 
+    sol['x'] = sol['x'][0]
+    sol['znl'], sol['snl'] = sol['znl'][1:], sol['snl'][1:]
     return sol
 
 
@@ -1835,7 +1978,7 @@ def gp(K, F, g, G=None, h=None, A=None, b=None):
                     G*x <= h      
                     A*x = b
 
-    Input arguments
+    Input arguments.
 
         K is a list of positive integers [K0, K1, K2, ..., Km].
 
@@ -1857,17 +2000,55 @@ def gp(K, F, g, G=None, h=None, A=None, b=None):
         zero rows.
 
 
-    Returns a dictionary with keys 'status', 'x', 'snl', 'sl, 'y', 
-        'znl', 'zl'.
+    Output arguments.
 
-        If status is 'optimal', x is the optimal solution, snl and sl
-        are the optimal slacks for the nonlinear and linear 
-        inequalities, znl and zl are the optimal dual veriables 
-        associated with the nonlinear and linear inequalities, and 
-        y are the optimal variables associated with the equality 
-        constraints.
+        Returns a dictionary with keys 'status', 'x', 'snl', 'sl',
+        'znl', 'zl', 'y', 'primal objective', 'dual objective', 'gap',
+        'relative gap', 'primal infeasibility', 'dual infeasibility',
+        'primal slack', 'dual slack'.
 
-        If status is 'unknown', x, snl, sl, y, znl and zl are None.
+        The 'status' field has values 'optimal' or 'unknown'.
+        If status is 'optimal', x, snl, sl, y, znl, zl  are approximate 
+        solutions of the primal and dual optimality conditions
+
+            f(x)[1:] + snl = 0,  G*x + sl = h,  A*x = b 
+            Df(x)'*[1; znl] + G'*zl + A'*y + c = 0 
+            snl >= 0,  znl >= 0,  sl >= 0,  zl >= 0
+            snl'*znl + sl'* zl = 0,
+
+        where fk(x) = log sum exp (Fk*x + gk). 
+
+        If status is 'unknown', x, snl, sl, y, znl, zl are the last
+        iterates before termination.  They satisfy snl > 0, znl > 0, 
+        sl > 0, zl > 0, but are not necessarily feasible.
+
+        The values of the other fields are the values returned by cpl()
+        applied to the epigraph form problem
+
+            minimize   t 
+            subjec to  f0(x) <= t
+                       fk(x) <= 0, k = 1, ..., mnl
+                       G*x <= h
+                       A*x = b.
+
+        Termination with status 'unknown' indicates that the algorithm 
+        failed to find a solution that satisfies the specified tolerances.
+        In some cases, the returned solution may be fairly accurate.  If
+        the primal and dual infeasibilities, the gap, and the relative gap
+        are small, then x, y, snl, sl, znl, zl are close to optimal.
+
+
+    Control parameters.
+
+       The following control parameters can be modified by adding an
+       entry to the dictionary options.
+
+       options['show_progress'] True/False (default: True)
+       options['maxiters'] positive integer (default: 100)
+       options['refinement'] nonnegative integer (default: 1)
+       options['abstol'] scalar (default: 1e-7)
+       options['reltol'] scalar (default: 1e-6)
+       options['feastol'] scalar (default: 1e-7).
     """
 
     import math 
