@@ -7,7 +7,7 @@ piecewise-linear objective and constraint functions.
 
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 1.0.
+# This file is part of CVXOPT version 1.1.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1425,7 +1425,7 @@ class _minmax(object):
                 elif 1 == lg != lgf: 
                     lg = lgf
 
-            if cnst: self._flist += [_function()+cnst]
+            if cnst is not None: self._flist += [_function()+cnst]
 
 
     def __len__(self):
