@@ -4,7 +4,7 @@ CVXOPT interface for MOSEK 5.0
 
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 1.0.
+# This file is part of CVXOPT version 1.1.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ def lp(c, G, h, A=None, b=None):
                     
     using MOSEK 5.0.
 
-    (solsta, x, y, z) = lp(c, G, h, A=None, b=None).
+    (solsta, x, z, y) = lp(c, G, h, A=None, b=None).
 
     Input arguments 
 
@@ -56,31 +56,29 @@ def lp(c, G, h, A=None, b=None):
     Return values
 
         solsta   the solution status.
-                 If solsta is solsta.optimal,
-                   then (x, y, z) contains the primal-dual solution.
-                 If solsta is solsta.prim_infeas_cer,
-                   then (x, y, z) is a certificate of primal infeasibility.
-                 If solsta is solsta.dual_infeas_cer,
-                   then (x, y, z) is a certificate of dual infeasibility.
-                 If solsta is solsta.unknown,
-                   then (x, y, z) are all None
+                 If solsta is solsta.optimal, then (x, y, z) contains the 
+                     primal-dual solution.
+                 If solsta is solsta.prim_infeas_cer, then (x, y, z) is a 
+                     certificate of primal infeasibility.
+                 If solsta is solsta.dual_infeas_cer, then (x, y, z) is a 
+                     certificate of dual infeasibility.
+                 If solsta is solsta.unknown, then (x, y, z) are all None.
 
                  Other return values for solsta include:  
-                   solsta.dual_feas  
-                   solsta.near_dual_feas
-                   solsta.near_optimal
-                   solsta.near_prim_and_dual_feas
-                   solsta.near_prim_feas
-                   solsta.prim_and_dual_feas
-                   solsta.prim_feas
+                     solsta.dual_feas  
+                     solsta.near_dual_feas
+                     solsta.near_optimal
+                     solsta.near_prim_and_dual_feas
+                     solsta.near_prim_feas
+                     solsta.prim_and_dual_feas
+                     solsta.prim_feas
                  in which case the (x,y,z) value may not be well-defined,
                  c.f., the MOSEK documentation.
         
         x, y, z  the primal-dual solution.                    
 
-    Options are passed to MOSEK solvers via the mosek.options
-    dictionary, e.g., the following turns off output from 
-    the MOSEK solvers
+    Options are passed to MOSEK solvers via the mosek.options dictionary. 
+    For example, the following turns off output from the MOSEK solvers
     
     >>> mosek.options = {iparam.log:0} 
     
@@ -185,6 +183,7 @@ def lp(c, G, h, A=None, b=None):
     else:
         return (solsta[0], x, z, y)
 
+
 def conelp(c, G, h, dims = None):
     """
     Solves a pair of primal and dual SOCPs
