@@ -1,6 +1,6 @@
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 1.0.
+# This file is part of CVXOPT version 1.1.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -190,5 +190,5 @@ def spmatrix_str_triplet(X):
     return s
 
 def spmatrix_repr_default(X):
-    return "<%ix%i matrix, tc='%c', nnz=%i>" \
+    return "<%ix%i sparse matrix, tc='%c', nnz=%i>" \
         %(X.size[0],X.size[1],X.typecode,len(X.V))
